@@ -3,35 +3,37 @@
 /*     Master list of function prototypes				*/
 /*----------------------------------------------------------------------*/
 
-/* from tclxcircuit.c: */
+/* from undo.c */
+
+extern void register_for_undo(u_int, objinstptr, ...);	/* variable argument list */
+extern void undo_action();
+extern void redo_action();
+extern void flush_undo_stack();
+extern void flush_redo_stack();
+extern void free_undo_record(Undoptr);
+extern void free_redo_record(Undoptr);
+
+
+/* from tclxcircuit.c */
 
 #ifdef TCL_WRAPPER
-
 extern Tk_EventProc *xctk_drag(ClientData, XEvent *);
 extern Tk_EventProc *xctk_draglscroll(ClientData, XEvent *);
 extern Tk_EventProc *xctk_dragfilebox(ClientData, XEvent *);
 extern int XcTagCallback(Tcl_Interp *, int, Tcl_Obj *CONST objv[]);
-extern objectptr NameToObject(char *, objinstptr *, Boolean);
+extern Tcl_Obj *evaluate_raw(oparamptr);
+extern char *TCL_to_PS(char *);
+#endif
 
-#endif /* TCL_WRAPPER */
-
-/* from the ASG module: */
-
-#ifdef ASG
-
-extern int asg_make_instance(Clientdata, char*, float, float, int, int, float);
-extern void asg_make_polygon(Clientdata, int, int x[], int y[]);
-extern void asg_make_label(Clientdata, u_char, int, int, char *);
-extern void importivf();
-extern void importspice();
-
-#endif /* ASG */
+extern char *evaluate_expr(oparamptr);
 
 /* from elements.c: */
 
+extern void copyalleparams(genericptr, genericptr);
 extern void copyparams(objinstptr, objinstptr);
 extern void objectdefaults(objinstptr, objectptr, int, int);
 extern void labeldefaults(labelptr, u_char, int, int);
+extern void poly_add_point(polyptr, XPoint *);
 extern void polydefaults(polyptr, int, int, int);
 extern void splinedefaults(splineptr, int, int);
 extern void arcdefaults(arcptr, int, int);
@@ -98,6 +100,7 @@ extern void panrefresh(u_int, XButtonEvent *);
 extern void centerpan(xcWidget, caddr_t, caddr_t);
 extern void checkwarp(XPoint *);
 extern int checkcycle(short, short);
+extern void copyvirtual();
 extern void nextpolycycle(polyptr, short);
 extern void nextsplinecycle(splineptr, short);
 extern void nextarccycle(arcptr, short);
@@ -136,10 +139,10 @@ extern void startrotate(xcWidget, pointertype, caddr_t);
 extern void startcopy(xcWidget, caddr_t, caddr_t);
 extern void startedit(xcWidget, caddr_t, caddr_t);
 extern void startdelete(xcWidget, caddr_t, caddr_t);
-extern void reviseselect(short *);
-extern void objectdelete(short);
+extern void reviseselect(short *, int, short *);
 extern void deletebutton(XButtonEvent *);
-extern short xc_undelete(xcWidget, pointertype, XButtonEvent *);
+extern short *xc_undelete(objinstptr, objectptr, short);
+extern objectptr delete_element(objinstptr, short *, int, short);
 extern void printname(objectptr);
 extern Boolean checkname(objectptr);
 extern objectptr finddot();
@@ -159,6 +162,9 @@ extern void releasebutton(XButtonEvent *);
 extern void pathbutton(genericptr *, XButtonEvent *);
 extern void resizearea(xcWidget, caddr_t, caddr_t);
 extern void drawarea(xcWidget, caddr_t, caddr_t);
+extern void standard_element_delete(short);
+extern void delete_for_xfer(short);
+extern void delete_noundo(short);
 
 /* from filelist.c: */
 
@@ -179,7 +185,7 @@ extern int lookdirectory(char *);
 
 /* from files.c: */
 
-extern void ridnewline(char *);
+extern char *ridnewline(char *);
 extern void free_single(genericptr *);
 extern void reset(objectptr, short);
 extern void pagereset(short);
@@ -191,9 +197,10 @@ extern short objcompare(objectptr, objectptr);
 extern float getpsscale(float, short);
 extern void dostcount(FILE *, short *, short);
 extern short printparams(FILE *, objinstptr, short);
+extern void printobjectparams(FILE *, objectptr);
 extern void varcheck(FILE *, short, objectptr, short *, genericptr, u_char);
 extern void varfcheck(FILE *, float, objectptr, short *, genericptr, u_char);
-extern void varpcheck(FILE *, short, objectptr, int, int, short *, genericptr, u_char);
+extern void varpcheck(FILE *, short, objectptr, int, short *, genericptr, u_char);
 extern void getfile(xcWidget, pointertype, caddr_t);
 extern Boolean nextfilename();
 extern void loadfontlib();
@@ -207,6 +214,7 @@ extern void startloadfile();
 extern void importfile();
 extern void loadfile(short);
 extern void readlabel(objectptr, char *, stringpart **);
+extern void readparams(objectptr, objinstptr, objectptr, char *);
 extern u_char *find_match(u_char *);
 extern char *advancetoken(char *);
 extern char *varpscan(objectptr, char *, short *, genericptr, int, int, u_char);
@@ -231,8 +239,10 @@ extern Boolean xc_tilde_expand(char *);
 extern short writelabel(FILE *, stringpart *, short *);
 extern char *writesegment(stringpart *, float *, int *);
 extern void writelabelsegs(FILE *, short *, stringpart *);
-extern void printobjects(FILE *, objectptr, float, float, float, short, short,
-                         objectptr **, short *, int);
+extern void printobjects(FILE *, objectptr, objectptr **, short *, int);
+extern void printrefobjects(FILE *, objectptr, objectptr **, short *);
+extern void printpageobject(FILE *, objectptr, short, short);
+
 
 /* from fontfile.c: */
 
@@ -259,6 +269,7 @@ extern void findsplinepos(splineptr, float, XPoint *, int *);
 extern void ffindsplinepos(splineptr, float, XfPoint *);
 extern float findsplinemin(splineptr, XPoint *);
 extern short closepoint(polyptr, XPoint *);
+extern void updateinstparam(objectptr);
 extern short checkbounds();
 extern void window_to_user(short, short, XPoint *);
 extern void user_to_window(XPoint, XPoint *);
@@ -306,12 +317,10 @@ extern void UDrawSimpleLine(XPoint *, XPoint *);
 extern void UDrawLine(XPoint *, XPoint *);
 extern void UDrawCircle(XPoint *, u_char);
 extern void UDrawX(labelptr);
-#ifdef SCHEMA
 extern void UDrawXDown(labelptr);
-extern int  toplevelwidth(objinstptr);
-extern int  toplevelheight(objinstptr);
+extern int  toplevelwidth(objinstptr, short *);
+extern int  toplevelheight(objinstptr, short *);
 extern void extendschembbox(objinstptr, XPoint *, XPoint *);
-#endif
 extern void pinadjust(short, short *, short *, short);
 extern void UDrawTextLine(labelptr, short);
 extern void UDrawTLine(labelptr);
@@ -326,6 +335,7 @@ extern void UDrawPolygon(polyptr);
 extern void UDrawArc(arcptr);
 extern void UDrawPath(pathptr);
 extern void UDrawObject(objinstptr, short, int, pushlistptr *);
+extern void TopDoLatex();
 
 /* from help.c: */
 
@@ -421,8 +431,11 @@ extern void makenewencodingbutton(char *, char);
 extern void togglefontmark(int);
 #ifndef TCL_WRAPPER
 extern void toggleexcl(xcWidget, menuptr, int);
+extern void promptparam(xcWidget, caddr_t, caddr_t);
 #endif
+extern void stringparam(xcWidget, caddr_t, caddr_t);
 extern void toggle(xcWidget, Boolean *, caddr_t);
+extern int setelementstyle(xcWidget, u_short, u_short);
 extern void setcolorscheme(Boolean);
 extern void inversecolor(xcWidget, Boolean *, caddr_t);
 extern void togglegrid(u_short);
@@ -439,11 +452,13 @@ extern void renamelib(short);
 extern void setcolormark(int);
 extern void setallstylemarks(u_short);
 extern polyptr checkforbbox(objectptr);
+#ifndef TCL_WRAPPER
 extern void setfill(xcWidget, pointertype, caddr_t);
 extern void makebbox(xcWidget, pointertype, caddr_t);
 extern void setclosure(xcWidget, pointertype, caddr_t);
-extern void setline(xcWidget, pointertype, caddr_t);
 extern void setopaque(xcWidget, pointertype, caddr_t);
+#endif
+extern int setline(xcWidget, pointertype, caddr_t);
 #ifdef TCL_WRAPPER
 extern void setcolor(xcWidget, int);
 #else
@@ -455,13 +470,11 @@ extern void setfontmarks(short, short);
 extern void startparam(xcWidget, pointertype, caddr_t);
 extern void startunparam(xcWidget, pointertype, caddr_t);
 extern void setdefaultfontmarks();
-extern void dopinvisible(xcWidget, labelptr);
-extern void doflipflip(xcWidget, labelptr);
+extern void setjustbit(xcWidget, pointertype, caddr_t);
+extern void setpinjustbit(xcWidget, pointertype, caddr_t);
 extern void setjust(xcWidget, pointertype, labelptr, short);
 extern void setvjust(xcWidget, pointertype, caddr_t);
 extern void sethjust(xcWidget, pointertype, caddr_t);
-extern void setflipinv(xcWidget, pointertype, caddr_t);
-extern void setpinvisible(xcWidget, pointertype, caddr_t);
 extern void boxedit(xcWidget, pointertype, caddr_t);
 extern void locloadfont(xcWidget, char *);
 extern void addnewfont(xcWidget, caddr_t, caddr_t);
@@ -478,14 +491,17 @@ extern void border_popup(xcWidget, caddr_t, caddr_t);
 extern void color_popup(xcWidget, caddr_t, caddr_t);
 extern void fill_popup(xcWidget, caddr_t, caddr_t);
 extern void param_popup(xcWidget, caddr_t, caddr_t);
-extern void dospecial(void);
+extern Boolean dospecial(void);
 
 /* from netlist.c: */
 
-#ifdef SCHEMA
-extern void createnets();
+extern Boolean RemoveFromNetlist(objectptr, genericptr);
+extern labelptr NetToLabel(int, objectptr);
+extern void NameToPosition(objinstptr, labelptr, XPoint *);
+extern void invalidate_netlist(objectptr);
+extern int updatenets(objinstptr);
+extern void createnets(objinstptr);
 extern void destroynets();
-extern void gennet(char *, char *);
 extern Boolean nonnetwork(polyptr);
 extern int globalmax();
 extern void gennetlist(objectptr, objinstptr);
@@ -507,7 +523,6 @@ extern void mergecleanup(objectptr);
 extern void mergenets(objectptr, int, int);
 extern void removecall(objectptr, CalllistPtr);
 extern void addpin(objectptr, labelptr, int);
-extern void addglobal(labelptr, int);
 extern void addcall(objectptr, objectptr, objinstptr);
 extern Boolean addport(objectptr, int);
 extern void addportcall(objectptr, objinstptr, objectptr, int, int);
@@ -523,37 +538,53 @@ extern void writeflat(objectptr, CalllistPtr, char *, FILE *, char *);
 extern void writeglobals(objectptr, FILE *);
 extern void writehierarchy(objectptr, CalllistPtr, FILE *);
 extern void writenet(objectptr, char *, char *);
-extern void writepcb(objectptr, CalllistPtr, char *, char *);
-extern void outputpcb(objectptr, FILE *);
-extern void freepcb();
+extern void writepcb(struct Ptab **, objectptr, CalllistPtr, char *, char *);
+extern void outputpcb(struct Ptab *, FILE *);
+extern void freepcb(struct Ptab *);
+extern void freepolylist(NetlistPtr);
+extern void freelabellist(NetlistPtr);
+extern void freeports(CalllistPtr);
 extern void freenets(objectptr);
 extern void freetemplabels(objectptr);
 extern void freeglobals();
-extern int cleartraversed(objectptr, int);
+extern int cleartraversed(objectptr);
 extern int checkvalid(objectptr);
 extern void clearpins(objectptr);
-#endif
 
 /* from parameter.c: */ 
 
+extern char *find_indirect_param(objinstptr, char *);
+extern oparamptr match_param(objectptr, char *);
+extern oparamptr match_instance_param(objinstptr, char *);
+extern oparamptr find_param(objinstptr, char *);
+extern int get_num_params(objectptr);
+extern void free_object_param(objectptr, oparamptr);
+extern void free_instance_param(objinstptr, oparamptr);
+extern void free_element_param(genericptr, eparamptr);
+
+extern oparamptr make_new_parameter(char *);
+extern eparamptr make_new_eparam(char *);
+
+extern void makeexprparam(objectptr, char *, char *, int);
+extern void makefloatparam(objectptr, char *, float);
+extern void makestringparam(objectptr, char *, stringpart *);
 extern void indicateparams(genericptr);
-extern void makenumericalp(genericptr *, u_int);
-extern void unmakenumericalp(genericptr *, u_int);
+extern void makenumericalp(genericptr *, u_int, char *);
 extern void noparmstrcpy(u_char *, u_char *);
 extern void insertparam();
-extern void makeparam(labelptr);
-extern void initinst(objectptr, objectptr, short);
-extern void searchinst(objectptr, objectptr, short);
-extern void initallinstances(objectptr, int);
+extern void makeparam(labelptr, char *);
+extern void searchinst(objectptr, objectptr, char *);
 extern stringpart *searchparam(stringpart *);
 extern void unmakeparam(labelptr, stringpart *);
+extern void unmakenumericalp(genericptr *, u_int);         
 extern void unparameterize(int);
-extern void parameterize(int);
+extern void parameterize(int, char *);
 extern genericptr findparam(objectptr, void *, u_char);
 extern Boolean paramcross(objectptr, labelptr);
 extern oparamptr parampos(objectptr, labelptr, char *, short *, short *);
 extern int opsubstitute(objectptr, objinstptr);
 extern int psubstitute(objinstptr);
+extern oparamptr copyparameter(oparamptr);
 extern void pwriteback(objinstptr);
 extern short paramlen(u_char *);
 extern int natstrlen(u_char *);
@@ -608,7 +639,8 @@ extern int reset_gs();
 
 /* from schema.c: */
 
-extern void callgennet(xcWidget, pointertype, caddr_t);
+extern objectptr NameToObject(char *, objinstptr *, Boolean);
+extern void callwritenet(xcWidget, pointertype, caddr_t);
 extern void doxschema(xcWidget, pointertype, caddr_t);
 extern void startconnect(xcWidget, caddr_t, caddr_t);
 extern int connectivity(xcWidget, caddr_t, caddr_t);
