@@ -156,10 +156,7 @@ typedef struct {
 #define MINAUTOSCALE 0.75 /* Won't automatically scale closer than this */
 #define MAXCHANGES 20 /* Number of changes to induce a temp file save	*/
 #define STIPPLES    8 /* Number of predefined stipple patterns		*/
-
-#ifdef SCHEMA
 #define PADSPACE   10 /* Spacing of pinlabels from their origins	*/
-#endif
 
 #ifdef HAVE_XPM
 #define TBBORDER   1  /* border around toolbar buttons */
@@ -203,13 +200,7 @@ CATPAN_MODE, CATALOG_MODE, CATTEXT_MODE, FONTCAT_MODE, FONTCAT2_MODE,
 TEXT1_MODE, TEXT2_MODE, TEXT3_MODE, WIRE_MODE, POLY_MODE, BOX0_MODE,
 BOX_MODE, EBOX_MODE, ARC0_MODE, ARC_MODE, EARC_MODE, SPLINE0_MODE,
 SPLINE_MODE, ESPLINE_MODE, EPATH_MODE, LPARAM_MODE, IPARAM_MODE,
-ULPARAM_MODE, UIPARAM_MODE
-
-#ifdef SCHEMA
-, CONNECT_MODE, ASSOC_MODE
-#endif
-
-};
+ULPARAM_MODE, UIPARAM_MODE, CONNECT_MODE, ASSOC_MODE};
 
 /*----------------------------------------------------------------------*/
 /* File loading modes							*/
@@ -341,8 +332,6 @@ ULPARAM_MODE, UIPARAM_MODE
 /* Schematic object types and pin label types	   			*/
 /*----------------------------------------------------------------------*/
 
-#ifdef SCHEMA
-
 #define PRIMARY 0		/* Primary (master) schematic page */
 #define SECONDARY 1		/* Secondary (slave) schematic page */
 #define TRIVIAL 2		/* Symbol as non-schematic element */
@@ -355,8 +344,6 @@ ULPARAM_MODE, UIPARAM_MODE
 
 #define HIERARCHY_LIMIT 256	/* Stop if recursion goes this deep */
 
-#endif
-
 /*----------------------------------------------------------------------*/
 /* Save types.	This list incorporates "ALL_PAGES", below.		*/
 /*----------------------------------------------------------------------*/
@@ -479,7 +466,7 @@ typedef struct _oparam {
    u_char	which;		/* what the parameter represents (P_*) */
    union {
       stringpart *string;	/* xcircuit label type */
-      char	 *expr;		/* TCL (postscript) expression */
+      char	 *expr;		/* TCL (string) expression */
       int	  ivalue;	/* also covers type short int by typecasting */
       float	  fvalue;
    } parameter;			/* default or substitution value */
@@ -537,9 +524,7 @@ typedef struct {
    objectptr	thisobject;
    oparamptr	params;		/* parameter substitutions for this instance */
    BBox		bbox;		/* per-instance bounding box information */
-#ifdef SCHEMA
    BBox		*schembbox;	/* Extra bounding box for pin labels */
-#endif
 } objinst, *objinstptr;
 
 /* Linked-list for objects */
@@ -584,9 +569,7 @@ typedef struct {
    short	rotation;
    float	scale;
    short	justify;
-#ifdef SCHEMA
    u_char	pin;
-#endif
    stringpart	*string;
 } label, *labelptr;
 
@@ -657,8 +640,6 @@ typedef struct {
 /* Netlist structures for schematic capture				*/
 /*----------------------------------------------------------------------*/
 
-#ifdef SCHEMA
-
 /* Linked polygon list */
 
 typedef struct _Polylist *PolylistPtr;
@@ -744,8 +725,6 @@ typedef struct {
    objinstptr	thisinst;
 } Highlight;
 
-#endif
-
 /*----------------------------------------------------------------------*/
 /* Main object structure						*/
 /*----------------------------------------------------------------------*/
@@ -762,7 +741,6 @@ typedef struct _xcobject {
    genericptr	*plist;	 	/* to be retypecast to label, polygon, etc. */
    oparamptr	params;		/* list of parameters, with default values */
    
-#ifdef SCHEMA
    Highlight	highlight;	/* net to be highlighted on redraw */
    u_char	schemtype;
    objectptr	symschem;	/* schematic page support */
@@ -772,7 +750,6 @@ typedef struct _xcobject {
    NetlistPtr   netlist;
    PortlistPtr  portlist;
    CalllistPtr  calllist;
-#endif
 } object;
 
 /*----------------------------------------------------------------------*/
@@ -905,10 +882,8 @@ enum {DIRECTORY = 0, MATCH, NONMATCH};	/* file types */
 /*----------------------------------------------------------------------*/
 
 typedef struct {
-#ifdef SCHEMA
    /* schematic layout colors */
    Pixel	globalcolor, localcolor, infocolor, ratsnestcolor;
-#endif
 
    /* non-schematic layout color(s) */
    Pixel	bboxpix;
@@ -1030,14 +1005,11 @@ enum {
    XCF_Pan_Down, XCF_Write, XCF_Rotate, XCF_Flip_X, XCF_Flip_Y, XCF_Snap,
    XCF_SnapTo, XCF_Pop, XCF_Push, XCF_Delete, XCF_Select, XCF_Box, XCF_Arc,
    XCF_Text, XCF_Exchange, XCF_Copy, XCF_Join, XCF_Unjoin, XCF_Spline,
-   XCF_Edit, XCF_Undelete, XCF_Select_Save, XCF_Unselect, XCF_Dashed,
+   XCF_Edit, XCF_Undo, XCF_Redo, XCF_Select_Save, XCF_Unselect, XCF_Dashed,
    XCF_Dotted, XCF_Solid, XCF_Prompt, XCF_Dot, XCF_Wire, XCF_Start,
    XCF_Finish, XCF_Cancel, XCF_Nothing, XCF_Exit,
-
-#ifdef SCHEMA
    XCF_Netlist, XCF_Swap, XCF_Pin_Label, XCF_Pin_Global, XCF_Info_Label,
    XCF_Connectivity, XCF_Sim, XCF_SPICE, XCF_PCB, XCF_SPICEflat,
-#endif
    NUM_FUNCTIONS
 };
 
@@ -1102,10 +1074,8 @@ typedef struct {
    Boolean	mapped;		/* indicates if window is drawable */
    Boolean	filefilter;	/* Is the file list filtered? */
    Boolean	editinplace;
-#ifdef SCHEMA
    Boolean	schemon;
    Boolean	pinpointon;
-#endif
 #ifdef HAVE_XPM
    Boolean	toolbar_on;
 #endif
@@ -1114,9 +1084,7 @@ typedef struct {
    XPoint	save, origin;
    short	selects;
    short	*selectlist;
-   short	lastselects;
    short	lastlibrary;
-   short	*selectlast;
    objinstptr	topinstance;
    objectptr	editstack;
    Matrixptr	MatStack;
@@ -1126,6 +1094,20 @@ typedef struct {
    char		*lastbackground;
 } Clientdata;
 
+/* Record for undo function */ 
+
+typedef struct _undostack *Undoptr;
+
+typedef struct _undostack {
+   Undoptr next;	/* next record in undo stack */
+   Undoptr last;	/* double-linked for "redo" function */
+   u_int type;		/* type of event */
+   objinstptr thisinst;	/* instance of object in which event occurred */
+   int idata;		/* simple undedicated integer datum */
+   char *undodata;	/* free space to be malloc'd */
+			/* (size dependent on "type") */
+} Undostack;
+
 /* This whole thing needs to be cleaned up. . . now I've got objects in */
 /* "library" which are repeated both in the "instlist" pair and the	*/
 /* "thisobject" pointer in each instance.  The instance is repeated on	*/
@@ -1151,7 +1133,8 @@ typedef struct {
    short	pages;
    short	numlibs;
    Pagedata	**pagelist;
-   Library	delbuffer;
+   Undoptr	undostack;	/* actions to undo */
+   Undoptr	redostack;	/* actions to redo */
    Library	fontlib;
    Library	*userlibs;
    objinstptr   *libtop;
